"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SDK = void 0;
const emitter_1 = require("@zodash/emitter");
const memoize_1 = require("@zodash/memoize");
const logger_1 = require("./services/logger");
const cache_1 = require("./services/cache");
const storage_1 = require("./services/storage");
const http_1 = require("./services/http");
const instance = memoize_1.memoize((_name, constructor, ...args) => new constructor(...args), (name) => name);
class SDK extends emitter_1.default {
    constructor() {
        super(...arguments);
        this._isReady = false;
        // functions
        //  auth
        //  user
    }
    get isReady() {
        return this._isReady;
    }
    // utils
    get logger() {
        return instance('logger', logger_1.default, 'core');
    }
    get cache() {
        return instance('cache', cache_1.default, 1e6);
    }
    get storage() {
        return instance('storage', storage_1.default);
    }
    // @TODO
    get http() {
        return http_1.default; // instance('storage', Storage);
    }
    //
    ready(fn) {
        return new Promise((resolve, reject) => {
            const onReady = (data) => {
                try {
                    this._isReady = true;
                    fn && fn.call(null, data);
                    resolve(data);
                }
                catch (error) {
                    reject(error);
                }
                finally {
                    this.off('ready', onReady);
                }
            };
            this.on('ready', onReady);
        });
    }
    close() {
        this.emit('close');
        return Promise.resolve();
    }
}
exports.SDK = SDK;
exports.default = SDK;
//# sourceMappingURL=index.js.map